<?php
// $Horde: horde/lib/Auth.php,v 1.13 2001/03/18 22:48:24 chuck Exp $

// Return codes
/** @constant AUTH_OK Operation succeeded. */
define('AUTH_OK', 0);

/** @constant AUTH_ERROR Operation failed. */
define('AUTH_ERROR', -1);

/** @constant AUTH_ERROR_PARAMS Bad or missing parameters. */
define('AUTH_ERROR_PARAMS', -2);

/** @constant AUTH_ERROR_CONNECT Connection failure. */
define('AUTH_ERROR_CONNECT', -3);

/** @constant AUTH_ERROR_AUTH Authentication failure. */
define('AUTH_ERROR_AUTH', -4);

/** @constant AUTH_ERROR_EMPTY Empty retrieval result. */
define('AUTH_ERROR_EMPTY', -5);

/** @constant AUTH_ERROR_UNSUPPORTED Method not supported by driver. */
define('AUTH_ERROR_UNSUPPORTED', -6);


/**
 * The Auth:: class provides a common abstracted interface into the
 * various backends for the Horde authentication system.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.13 $
 * @since   Horde 1.3
 * @package horde.auth
 */
class Auth {
    
    /**
     * Attempts to return a concrete Auth instance based on $driver.
     * 
     * @param string $driver The type of concrete Auth subclass to return.
     *                       This is based on the storage driver ($driver). The
     *                       code is dynamically included.
     * @param array  $params (optional) A hash containing any additional
     *                       configuration or connection parameters a subclass
     *                       might need.
     *
     * @return object Auth   The newly created concrete Auth instance, or false
     *                       on an error.
     */
    function factory($driver, $params = array())
    {
        $driver = strtolower($driver);

        if (empty($driver) || (strcmp($driver, 'none') == 0)) {
            return new Auth;
        }

        if (@include_once dirname(__FILE__) . '/Auth/' . $driver . '.php') {
            $class = 'Auth_' . $driver;
            return new $class($params);
        } else {
            return false;
        }
    }
    
    /**
     * Attempts to return a reference to a concrete Auth instance based on
     * $driver. It will only create a new instance if no Auth instance
     * with the same parameters currently exists.
     *
     * This should be used if multiple permissions sources (and, thus,
     * multiple Auth instances) are required.
     *
     * This method must be invoked as: $var = &Auth::singleton()
     *
     * @param string $driver The type of concrete Auth subclass to return.
     *                       This is based on the storage driver ($driver). The
     *                       code is dynamically included.
     * @param array  $params (optional) A hash containing any additional
     *                       configuration or connection parameters a subclass
     *                       might need.
     *
     * @return object Auth  The concrete Auth reference, or false on an error.
     */
    function &singleton($driver, $params = array())
    {
        static $instances;

        if (!isset($instances)) $instances = array();

        $signature = md5(strtolower($driver) . '][' . implode('][', $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = Auth::factory($driver, $params);
        }

        return $instances[$signature];
    }

    /**
     * Find out if a set of login credentials are valid.
     * 
     * @param string $userID       The userID to check
     * @param array  $credentials  The credentials to check
     * 
     * @return boolean Whether or not the credentials are valid.
     */
    function authenticate($userID, $credentials)
    {
        return Auth::checkAuth($userID);
    }

    /**
     * Add a set of authentication credentials.
     * 
     * @param string $userID       The userID to add.
     * @param array  $credentials  The credentials to use.
     */
    function add($userID, $credentials) {}

    /**
     * Update a set of authentication credentials.
     * 
     * @param string $oldID        The old userID.
     * @param string $newID        The new userID.
     * @param array  $credentials  The new credentials
     */
    function update ($oldID, $newID, $credentials) {}

    /**
     * Delete a set of authentication credentials.
     * 
     * @param string $userID  The userID to delete.
     */
    function delete ($userID) {}
    
    /**
     * Checks if there is a session with valid auth information.
     *
     * @param string $userID          The userID we are expecting in the session.
     * @param optional string $realm  The authentication realm to check for the user.
     *
     * @return boolean                Whether or not the user is authenticated.
     */
    function checkAuth($userID, $realm = null)
    {
        if (!empty($realm)) {
            if (isset($GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm])) {
                if (!empty($GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm]['authenticated']) &&
                    !empty($GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm]['userID']) &&
                    $GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm]['userID'] == $userID) {
                    return true;
                }
            }
        } else {
            if (isset($GLOBALS['HTTP_SESSION_VARS']['__auth'])) {
                if (!empty($GLOBALS['HTTP_SESSION_VARS']['__auth']['authenticated']) &&
                    !empty($GLOBALS['HTTP_SESSION_VARS']['__auth']['userID']) &&
                    $GLOBALS['HTTP_SESSION_VARS']['__auth']['userID'] == $userID) {
                    return true;
                }
            }
        }
        return false;
    }
	
    /**
     * Return the currently logged in user, if there is one.
     *
     * @param optional string $realm  The authentication realm to check for a user.
     * 
     * @return mixed $userID The userID of the current user, or false
     *                       if no user is logged in.
     */
    function getAuth($realm = null)
    {
        if (!empty($realm)) {
            if (isset($GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm])) {
                if (!empty($GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm]['authenticated']) &&
                    !empty($GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm]['userID'])) {
                    return $GLOBALS['HTTP_SESSION_VARS']['__auth'][$realm]['userID'];
                }
            }
        } else {
            if (isset($GLOBALS['HTTP_SESSION_VARS']['__auth'])) {
                if (!empty($GLOBALS['HTTP_SESSION_VARS']['__auth']['authenticated']) &&
                    !empty($GLOBALS['HTTP_SESSION_VARS']['__auth']['userID'])) {
                    return $GLOBALS['HTTP_SESSION_VARS']['__auth']['userID'];
                }
            }
        }
        return false;
    }

    /**
     * Set a variable in the session saying that authorization has
     * succeeded, note which userID was authorized, and note when the
     * login took place.
     *
     * @param string $userID  The userID who has been authorized.
     */
    function setAuth($userID)
    {
        if (!isset($GLOBALS['HTTP_SESSION_VARS']['__auth'])) {
            session_register('__auth');
        }
        $GLOBALS['__auth'] = &$GLOBALS['HTTP_SESSION_VARS']['__auth'];
        $GLOBALS['__auth'] = array();
        $GLOBALS['__auth']['authenticated'] = true;
        $GLOBALS['__auth']['userID'] = $userID;
        $GLOBALS['__auth']['timestamp'] = time();
    }
    
    /**
     * Clear any authentication tokens in the current session.
     */
    function clearAuth()
    {
        if (isset($GLOBALS['HTTP_SESSION_VARS']['__auth'])) {
            $GLOBALS['__auth'] = &$GLOBALS['HTTP_SESSION_VARS']['__auth'];
            $GLOBALS['__auth'] = array();
            $GLOBALS['__auth']['authenticated'] = null;
        }
    }
    
}
?>
