<?php
// $Horde: horde/lib/Auth/ldap.php,v 1.1 2001/02/22 16:26:58 chuck Exp $

Horde::functionCheck('ldap_connect', true,
    'Auth_ldap: Required LDAP functions were not found.');

/**
 * The Auth_ldap class provides an LDAP implementation of the Horde
 * authentication system.
 *
 * Required values for $params:
 *      'hostspec'      The hostname of the LDAP server.
 *      'basedn'        The base DN for the LDAP server.
 *      'uid'           The username search key.
 *
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.1 $
 * @since   Horde 1.3
 * @package horde.auth
 */
class Auth_ldap extends Auth {

    /**
     * Hash containing connection parameters.
     * @var array $params
     */
    var $params = array();


    /**
     * Constructs a new LDAP permissions object.
     *
     * @param array $params   A hash containing connection parameters.
     */
    function Auth_ldap($params = array())
    {
        if (isset($params['hostspec'])) {
            $this->params['hostspec'] = $params['hostspec'];
        }
        if (isset($params['basedn'])) {
            $this->params['basedn'] = $params['basedn'];
        }
        if (isset($params['uid'])) {
            $this->params['uid'] = $params['uid'];
        }
    }


    /**
     * Find out if the given set of login credentials are valid.
     * 
     * @param string $userID       The userID to check.
     * @param array  $credentials  An array of login credentials.
     * 
     * @return boolean Whether or not the credentials are valid.
     */
    function authenticate($userID, $credentials)
    {
        if (Auth::checkAuth($userID)) {
            return true;
        }
        
        /* Ensure we've been provided with all of the necessary parameters. */
        if (!isset($this->params['hostspec'])) return AUTH_ERROR_PARAMS;
        if (!isset($this->params['basedn'])) return AUTH_ERROR_PARAMS;
        if (!isset($this->params['uid'])) return AUTH_ERROR_PARAMS;
        if (!isset($credentials['password'])) return AUTH_ERROR_PARAMS;

        /* Connect to the LDAP server. */
        $ldap = @ldap_connect($this->params['hostspec']);
        if (!$ldap) return AUTH_ERROR_CONNECT;

        /* Search for the user's full DN. */
        $search = @ldap_search($ldap, $this->params['basedn'],
            $this->params['uid'] . '=' . $userID, array($this->params['uid']));
        $result = @ldap_get_entries($ldap, $search);
        if (is_array($result) && (count($result) > 1)) {
            $dn = $result[0]['dn'];
        } else {
            return AUTH_ERROR_EMPTY;
        }

        /* Attempt to bind to the LDAP server as the user. */
        $bind = @ldap_bind($ldap, $dn, $credentials['password']);
        if ($bind != false) {
            @ldap_close($ldap);
            Auth::setAuth($userID);
            return true;
        }

        @ldap_close($ldap);
        return false;
    }


    /**
     * Add a set of authentication credentials. Not yet implemented for LDAP.
     * 
     * @param string $userID       The userID to add.
     * @param array  $credentials  The credentials to be set.
     */
    function add($userID, $credentials)
    {
        return AUTH_ERROR_UNSUPPORTED;
    }


    /**
     * Update a set of authentication credentials. Not yet implemented for LDAP.
     * 
     * @param string $oldID        The old userID.
     * @param string $newID        The new userID.
     * @param array  $credentials  The new credentials
     */
    function update($oldID, $newID, $credentials)
    {
        return AUTH_ERROR_UNSUPPORTED;
    }


    /**
     * Delete a set of authentication credentials. Not yet implemented for LDAP.
     * 
     * @param string $userID The userID to delete.
     */
    function delete($userID)
    {
        return AUTH_ERROR_UNSUPPORTED;
    }
}

?>
