<?php
// $Horde: horde/lib/Horde.php,v 1.97 2001/06/04 21:54:47 chuck Exp $

// Horde Common Actions
/** @constant NO_ACTION The null action/noop. */ define('NO_ACTION', 0);
/** @const UPDATE_PREFS Update preferences.   */ define('UPDATE_PREFS', 1);

// Horde Framework Actions
/** @constant HORDE_LOGIN The login action.                */ define('HORDE_LOGIN', 50);
/** @constant HORDE_SEND_MESSAGE The email-sending action. */ define('HORDE_SEND_MESSAGE', 51);

// Horde Message Types
/** @constant HORDE_MESSAGE An informational message. */ define('HORDE_MESSAGE', 0);
/** @constant HORDE_SUCCESS A success message.        */ define('HORDE_SUCCESS', 1);
/** @constant HORDE_WARNING A warning.                */ define('HORDE_WARNING', 2);
/** @constant HORDE_ERROR Unrecoverable failure.      */ define('HORDE_ERROR', 4);


/**
 * If called with a filename, register that filename to be deleted at
 * request shutdown. If called with no arguments, unlink all files
 * registered. The first time it is called, it initializes the array
 * of files, and registers itself as a shutdown function - no need to
 * do so manually.
 *
 * @param string (optional) The filename to be deleted at the end of
 *                          the request.
 */
function _fileCleanup($filename = false)
{
    static $files;
    
    if (!isset($files) || !is_array($files)) {
        $files = array();
        register_shutdown_function('_fileCleanup');
    }
    
    if (!$filename) {
        foreach ($files as $file) {
            if (@file_exists($file))
                @unlink($file);
        }
    } else {
        $files[] = $filename;
    }
}

/**
 * The Horde:: class provides the functionality shared by all Horde
 * applications.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.97 $
 * @since   Horde 1.3
 * @package horde
 */
class Horde {
    
    /**
     * Add a message to the Horde message stack.
     *
     * @access public
     *
     * @param string $message The text description of the message.
     * @param int    $type    The type of message: HORDE_ERROR,
     *                        HORDE_WARNING, HORDE_SUCCESS, or HORDE_MESSAGE.
     */
    function raiseMessage($message, $type = HORDE_MESSAGE)
    {
        global $hordeMessageStack;
        $hordeMessageStack[] = array('type' => $type, 'message' => $message);
    }

    /**
     * Check the existence of the given function.
     *
     * @access public
     *
     * @param string  $function  The name of the function to test.
     * @param boolean $fatal     (optional) Exit fatally if $function doesn't
     *                           exist.
     * @param boolean $complaint (optional) Display an error message if
     *                           $function doesn't exist.
     *
     * @return boolean The availability of $function.
     */
    function functionCheck($function, $fatal = false, $complaint = '')
    {
        if (function_exists($function)) {
            return true;
        } else {
            if (!empty($complaint)) {
                echo "<h2>$complaint</h2>\n";
            }
            if ($fatal) {
                exit;
            }
            return false;
        }
    }
    
    /**
     * Return a session-id-ified version of $uri.
     *
     * @param string  $uri The URI to be modified.
     * @param boolean $full Generate a full (http://server/path/) URL.
     * @param boolean $always_append_session Tack on the session ID even if
     *                                       cookies are present.
     *
     * @return string The url with the session id appended
     */
    function url($uri, $full = false, $always_append_session = false)
    {
        $protocol = 'http';
        
        if ($full) {
            global $conf, $HTTP_SERVER_VARS;
            
            if (!isset($conf['use_ssl'])) {
                $conf['use_ssl'] = 2;
            }
            
            if ($conf['use_ssl'] == 1) {
                $protocol = 'https';
            } else if ($conf['use_ssl'] == 2) {
                if ((isset($HTTP_SERVER_VARS['HTTPS']) && $HTTP_SERVER_VARS['HTTPS'] == 'on') || getenv('SSL_PROTOCOL_VERSION')) {
                    $protocol = 'https';
                }
            }
            
            if (($HTTP_SERVER_VARS['SERVER_PORT'] != 80) && ($HTTP_SERVER_VARS['SERVER_PORT'] != 443)) {
                $server = $HTTP_SERVER_VARS['SERVER_NAME'] . ':' . $HTTP_SERVER_VARS['SERVER_PORT'];
            } else {
                $server = $HTTP_SERVER_VARS['SERVER_NAME'];
            }
            
            $url = $protocol . '://' . $server;
            if (substr($uri, 0, 1) != '/') {
                if (substr($conf['paths']['root'], -1) == '/') {
                    $url .= $conf['paths']['root'] . $uri;
                } else {
                    $url .= $conf['paths']['root'] . '/' . $uri;
                }
            } else {
                $url .= $uri;
            }
        } else {
            $url = $uri;
        }
        
        if ($always_append_session ||
            !isset($GLOBALS['HTTP_COOKIE_VARS'][session_name()])) {
            $url .= (strpos($url, '?') != false ?  '&' : '?') .
                urlencode(session_name()) . '=' . session_id();
        }
        
        return ($full ? $url : htmlentities($url));
    }
    
    /**
     * Return a session-id-ified version of $uri, using the current
     * application's $conf['paths']['root'] setting.
     *
     * @param string  $uri The URI to be modified.
     * @param boolean $full Generate a full (http://server/path/) URL.
     * @param boolean $always_append_session Tack on the session ID even if
     *                                       cookies are present.
     *
     * @return string The url with the session id appended
     */
    function applicationUrl($uri, $full = false, $always_append_session = false)
    {
        global $conf;
        
        if ($full || empty($conf['paths']['root'])) {
            return Horde::url($uri, $full, $always_append_session);
        } elseif (substr($conf['paths']['root'], -1) == '/') {
            return Horde::url($conf['paths']['root'] . $uri, $full, $always_append_session);
        } else {
            return Horde::url($conf['paths']['root'] . '/' . $uri, $full, $always_append_session);
        }
    }
    
    /**
     * Return an anchor tag with the relevant parameters 
     *
     * @param $url The full URL to be linked to
     * @param $status An optional JavaScript mouse-over string
     * @param $class The CSS class of the link
     * @param $target The window target to point this link too
     *
     * @return the full <a href> tag
     */
    function link($url, $status = false, $class = false, $target = false)
    {
        $ret = "<a href=\"$url\"";
        if ($status) {
            $ret .= ' onmouseout="status=\'\';" onmouseover="status=\''.
                htmlentities(addslashes($status)) . '\'; return true;"';
        }
        if ($class) {
            $ret .= " class=\"$class\"";
        }
        if ($target) {
            $ret .= " target=\"$target\"";
        }
        
        return "$ret>";
    }

    /**
     * Print an anchor tag with the relevant parameters
     *
     * @param $url The full URL to be linked to
     * @param $status An optional JavaScript mouse-over string
     * @param $class The CSS class of the link
     * @param $target The window target to point this link too
     */
    function plink($url, $status = false, $class = false, $target = false)
    {
       echo Horde::link($url, $status, $class, $target);
    }

    /**
     * Print a session-id-ified version of the URI.
     *
     * @param $uri the URI to be modified
     * @param $full Generate a full (http://server/path/) URL.
     * @param $always_append_session Tack on the session ID even if
     *                               cookies are present.
     */
    function purl($uri, $full = false, $always_append_session = false)
    {
        echo Horde::url($uri, $full, $always_append_session);
    }

    /**
     * Return a session-id-ified version of $PHP_SELF.
     *
     * @param $query_string (optional) include the query string?
     */
    function selfURL($query_string = false)
    {
        global $HTTP_SERVER_VARS;
        
        $url = $HTTP_SERVER_VARS['SCRIPT_NAME'];
        
        if ($query_string && !empty($HTTP_SERVER_VARS['QUERY_STRING'])) {
            $url .= '?' . $HTTP_SERVER_VARS['QUERY_STRING'];
        }
        
        return Horde::url($url);
    }

    /**
     * Print a session-id-ified version of $PHP_SELF.
     *
     * @param $query_string (optional) include the query string?
     */
    function pselfURL($query_string = false)
    {
        echo Horde::selfURL($query_string);
    }

    /**
     * Return a hidden form input containing the session name and id.
     */
    function formInput()
    {
        return '<input type="hidden" name="' . session_name() . '" value="' . session_id() . '" />';
    }

    /**
     * Print a hidden form input containing the session name and id.
     */
    function pformInput()
    {
        echo Horde::formInput();
    }
    
    
    //
    // Image utility functions
    //
    
    /**
     * Construct a correctly-pathed link to an image
     *
     * @param string $src  The image file.
     * @param string $attr Any extra parameters for the image tag.
     * @param string $dir  The root graphics directory.
     *
     * @return string The full image tag.
     */
    function img($src, $attr = '', $dir = null)
    {
        if (!isset($dir)) {
            global $conf;
            if (isset($conf['paths']['graphics']))
                $dir = $conf['paths']['graphics'];
            elseif (isset($conf['horde']['paths']['graphics']))
                $dir = $conf['horde']['paths']['graphics'];
            else
                $dir = 'graphics';
        }
        
        $img = '<img';
        $img .= ' src="' . $dir . '/' . $src . '"';
        $img .= ' border="0"';
        if (!empty($attr)) $img .= ' ' . $attr;
        $img .= ' />';
        
        return $img;
    }
    
    /**
     * Construct a correctly-pathed link to an image
     *
     * @param string $src  The image file.
     * @param string $attr Any extra parameters for the image tag.
     * @param string $dir  The root graphics directory.
     */
    function pimg ($src, $attr = null, $dir = null) { echo Horde::img($src, $attr, $dir); }
    
    /**
     * If magic_quotes_gpc is in use, run stripslashes() on $var.
     *
     * @param  string $var The string to un-quote, if necessary.
     * @return string      $var, minus any magic quotes.
     */
    function dispelMagicQuotes($var)
    {
        static $magic_quotes;
        
        if (!isset($magic_quotes)) {
            $magic_quotes = get_magic_quotes_gpc();
        }
        
        if ($magic_quotes) {
            if (!is_array($var)) {
                $var = stripslashes($var);
            }
        }
        
        return $var;
    }
    
    /**
     * Get a form variable from GET or POST data, stripped of magic
     * quotes if necessary.
     *
     * @param string $var The name of the form variable to look for.
     * @param string $default (optional) The value to return if the
     *                                   variable is not there.
     *
     * @return string     The cleaned form variable, or $default.
     */
    function getFormData($var, $default = null)
    {
        if (isset($GLOBALS['HTTP_POST_VARS'][$var])) {
            return Horde::dispelMagicQuotes($GLOBALS['HTTP_POST_VARS'][$var]);
        } elseif (isset($GLOBALS['HTTP_GET_VARS'][$var])) {
            return Horde::dispelMagicQuotes($GLOBALS['HTTP_GET_VARS'][$var]);
        } else {
            return $default;
        }
    }

    /**
     * Determine the location of the system temporary directory.
     * If a specific setting cannot be found, it defaults to /tmp
     *
     * @return string    A directory name which can be used for temp files
     *                   or false if one could not be found
     */
    function getTempDir()
    {
        $tmp_locations = array('/tmp', '/var/tmp', 'c:\temp');

        /* If one has been specifically set, then use that */
        if (!empty($GLOBALS['conf']['tmpdir'])) {
            $tmp = $GLOBALS['conf']['tmpdir'];
        }
        
        /* Next, try PHP's upload_tmp_dir directive. */
        if (empty($tmp)) {
            $tmp = ini_get('upload_tmp_dir');
        }

        /* Otherwise, try to determine the TMPDIR environment
           variable. */
        if (empty($tmp)) {
            $tmp = getenv('TMPDIR');
        }

        /* If we still cannot determine a value, then cycle through a
         * list of preset possibilities. */
        while (empty($tmp) && sizeof($tmp_locations)) {
            $tmp_check = array_shift($tmp_locations);
            if (@is_dir($tmp_check)) {
                $tmp = $tmp_check;
            }
        } 

        /* If it is still empty, we have failed, so return false;
         * otherwise return the directory determined. */
        return empty($tmp) ? false : $tmp;
    }

    /**
     * Create a temporary filename for the lifetime of the script, and
     * (optionally) register it to be deleted at request shutdown.
     *
     * @param string $prefix  Prefix to make the temporary name more recognizable
     * @param optional boolean $delete Delete the file at the end of the request?
     * @return string         containing the full path-name to the temporary file
     *                        or false if a temp file could not be created
     */
    function getTempFile($prefix = 'Horde', $delete = true)
    {
        $tmp_dir = Horde::getTempDir();

        if (empty($tmp_dir)) {
            return false;
        }
        
        $tmp_file = tempnam($tmp_dir, $prefix);

        /* If the file was created, then register it for deletion and return */
        if (empty($tmp_file)) {
            return false;
        } else {
            if ($delete) {
                _fileCleanup($tmp_file);
            }
            return $tmp_file;
        }
    }

}
?>
