<?php
/**
 * The Lang:: class provides common methods for handling language detection
 * and selection.
 *
 * @author  Jon Parise <jon@horde.org>
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.8 $
 * @since   Horde 1.3
 * @package horde
 */
class Lang {

    /**
     * Selects the most preferred language for the current client session.
     *
     * @param optional boolean $set_cookie   Boolean determining whether a
     *                                       cookie containing the selected
     *                                       language should be sent.
     *
     * @return string        The selected language abbreviation.
     */
    function select($set_cookie = false)
    {
        global $HTTP_SERVER_VARS, $HTTP_COOKIE_VARS, $conf;
    
        /* Use a directly supplied parameter, if given. */
        $language = Horde::getFormData('language');
        if (isset($language) && Lang::isValid($language)) {
            // We already have a valid language; no more work needed.

        /* If we have already set a cookie, simply use that. */
        } elseif (isset($HTTP_COOKIE_VARS['horde_language']) && Lang::isValid($HTTP_COOKIE_VARS['horde_language'])) {
            $language = $HTTP_COOKIE_VARS['horde_language'];

        /* No browser cookie, try browser-accepted languages, then default. */
        } elseif (isset($HTTP_SERVER_VARS['HTTP_ACCEPT_LANGUAGE'])) {

            /* Start with a known default value. */
            $language = $conf['lang']['default'];

            /* The browser supplies a list, so return the first valid one. */
            $browser_langs = explode(',', $HTTP_SERVER_VARS['HTTP_ACCEPT_LANGUAGE']);
            foreach ($browser_langs as $lang) {
                $lang = trim($lang);
                if (Lang::isValid($lang)) {
                    $language = $lang;
                    break;
                } elseif (Lang::isValid(substr($lang, 0, 2))) {
                    $language = substr($lang, 0, 2);
                    break;
                }
            }

        /* No dice auto-detecting, so give them the server default. */
        } else {
            $language = $conf['lang']['default'];
        }

        /* Set the language cookie to expire in a year. */
        if ($set_cookie) {
            setcookie('horde_language', $language, time() + 31536000, '/', '', 0);
        }

        return basename($language);
    }

    /**
     * Determines whether the supplied language is valid.
     *
     * @param $language         The abbreviated name of the language.
     *
     * @return  boolean         True if the language is valid, false if it's
     *                          not valid or unknown.
     */
    function isValid($language)
    {
        return !empty($GLOBALS['nls']['languages'][$language]);
    }
    
}
?>
