<?php
// $Horde: horde/lib/MIME/Viewer.php,v 1.5 2001/05/27 18:59:21 avsm Exp $

/**
 * The MIME_Viewer:: class provides an abstracted interface to 
 * render out MIME types into HTML format.  It depends on a
 * set of MIME_Viewer_* drivers which handle the actual rendering,
 * and also a configuration file to map MIME types to drivers.
 *
 * @author  Anil Madhavapeddy <anil@recoil.org>
 * @version $Revision: 1.5 $
 * @since   Horde 1.3
 * @package horde.mime.viewer
 */

class MIME_Viewer {

    var $mime_part;
    
    /**
     * Attempts to return a concert MIME_Viewer_* object based on the 
     * type of MIME_Part passed onto it.
     *
     * @param &MIME_Part Reference to a MIME_Part object with the information to be rendered
     */
    function factory(&$mime_part) {

        global $mime_drivers_map, $mime_drivers;

        /* Check that we have a valid MIME_Part object */
        if (!is_object($mime_part) || get_class($mime_part) != 'mime_part') {
            return false;
        }

        /* Determine driver type from the MIME type */
        $mime_type = $mime_part->getType();
        if (!$mime_type) {
            return false;
        }

        $driver = MIME_Viewer::getDriver($mime_type);

        /* Spawn the relevant driver, and return it (or false on failure) */
        if (@include_once dirname(__FILE__) . '/Viewer/' . $driver . '.php') {
            $class = 'MIME_Viewer_' . $driver;
            return new $class($mime_part);
        } else {
            return false;
        }

    }

    /**
     * Constructor for MIME_Viewer
     *
     * @param &MIME_Part    Reference to a MIME_Part object with the information to be rendered
     */
    function MIME_Viewer(&$mime_part) {
        $this->mime_part = &$mime_part;
    }

    /**
     * Return the MIME type of the rendered content.  This can be overridden
     * by the individual drivers, depending on what format they output in.
     * By default, it passes through the MIME type of the object, or replaces
     * custom extension types with 'text/plain' to let the browser do a best-guess
     * render.
     *
     * @return string MIME-type of the output content
     */
    function getType() {
        if ($this->mime_part->type == 'x-extension') {
            return 'text/plain';
        } else {
            return $this->mime_part->getType();
        }
    }

    /**
     * Return the rendered version of the object.  Should be overridden
     * by individual drivers to perform custom tasks.
     *
     * The $mime_part class variable has the information to render,
     * encapsulated in a MIME_Part object.
     * @return string Rendered version of the object
     */
    function render() {
        return $this->mime_part->getContents();
    }

    /**
     * Given an input MIME type, this static function resolves it
     * into a specific output driver which can handle it.
     * @param $mimeType string MIME type to resolve
     * @return string Name of driver (e.g. 'enscript')
     */
    function getDriver($mimeType) {
        global $mime_drivers, $mime_drivers_map;

        $driver = 'default';

        /* If an override exists for this MIME type, then use that */
        if (isset($mime_drivers_map['overrides'][$mimeType])) {
            $driver = $mime_drivers_map['overrides'][$mimeType];

        } else {
            /* Iterate through the list of registered drivers, and see if
             * this MIME type exists in the MIME types that they claim to handle.
             * If the driver handles it, then assign it as the rendering driver */
            foreach ($mime_drivers_map['registered'] as $dr) {
                if (in_array($mimeType, $mime_drivers[$dr]['handles'])) {
                    $driver = $dr;
                }
            }
        }
        
        return $driver;
    }

    /**
     * Given an input MIME type, this static function returns
     * the name of an icon that can be associated with it
     * @param $mimeType string MIME type to get the icon for
     * @return string Icon name to use
     */
    function getIcon($mimeType) {
        global $mime_drivers;

        $driver = MIME_Viewer::getDriver($mimeType);

        /* If a specific icon for this driver and mimetype is defined,
         * then use that */
        if (!isset($mime_drivers[$driver]['icons'][$mimeType])) {
            /* If a default icon for this driver exists, use that */
            if (!isset($mime_drivers[$driver]['icons']['default'])) {
                return $mime_drivers['default']['icons']['default'];
            } else {
                return $mime_drivers[$driver]['icons']['default'];
            }
        } else {
            return $mime_drivers[$driver]['icons'][$mimeType];
        }
    }

}

?>
