<?php
// $Horde: horde/lib/Prefs/sql.php,v 1.17 2001/05/27 16:10:56 chuck Exp $

/**
 * Preferences storage implementation for PHP's PEAR database abstraction
 * layer.
 *
 * Required values for $params:
 *      'phptype'       The database type (ie. 'pgsql', 'mysql, etc.).
 *      'hostspec'      The hostname of the database server.
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'database'      The name of the database.
 *      'table'         The name of the preferences table in 'database'.
 * 
 * Required by some database implementations:
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.
 *
 * The table structure for the preferences is as follows:
 *
 *  create table horde_prefs (
 *      uid             varchar(32) not null,
 *      scope           varchar(16) not null default '',
 *      pref_name       varchar(32) not null,
 *      pref_value      text null,
 *      primary key (uid, scope, pref_name)
 *  );
 *
 * @author  Jon Parise <jon@csh.rit.edu>
 * @version $Revision: 1.17 $
 * @since   Horde 1.3
 * @package horde.prefs
 */
class Prefs_sql extends Prefs {

    /** Hash containing connection parameters. */
    var $params = array();

    /** Handle for the current database connection.
        @var object DB $db */
    var $db;

    /** Boolean indicating whether or not we're connected to the SQL server. */
    var $connected = false;


    /**
     * Constructs a new SQL preferences object.
     *
     * @param string $user      The user who owns these preferences.
     * @param string $password  The password associated with $user. (unused)
     * @param string $scope     The current preferences scope.
     * @param array  $params    A hash containing connection parameters.
     * @param boolean $caching  (optional) Should caching be used?
     */
    function Prefs_sql($user, $password = '', $scope = '', $params = array(),
        $caching = false)
    {
        $this->user = $user;
        $this->scope = $scope;
        $this->params = $params;
        $this->caching = $caching;
    }

    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return int         PREFS_OK on success, PREFS_ERROR_* on failure.
     */
    function connect()
    {
        if (!$this->connected) {
            include_once 'DB.php';

            if (!is_array($this->params)) return PREFS_ERROR_PARAMS;
            if (!isset($this->params['phptype'])) return PREFS_ERROR_PARAMS;
            if (!isset($this->params['hostspec'])) return PREFS_ERROR_PARAMS;
            if (!isset($this->params['username'])) return PREFS_ERROR_PARAMS;
            if (!isset($this->params['password'])) return PREFS_ERROR_PARAMS;
            if (!isset($this->params['database'])) return PREFS_ERROR_PARAMS;
            if (!isset($this->params['table'])) return PREFS_ERROR_PARAMS;

            /* Connect to the SQL server using the supplied parameters. */
            $this->db = &DB::connect($this->params, true);
            if (DB::isError($this->db) || DB::isWarning($this->db)) {
                return PREFS_ERROR_CONNECT;
            }

            $this->connected = true;
        }

        return PREFS_OK;
    }

    /**
     * Disconnect from the SQL server and clean up the connection.
     *
     * @return boolean     true on success, false on failure.
     */
    function disconnect()
    {
        if ($this->connected) {
            $this->connected = false;
            return $this->db->disconnect();
        }

        return true;
    }

    /**
     * Retrieves the requested set of preferences from the user's database
     * entry.
     *
     * @param optional array $prefs  An array listing the preferences to
     *                     retrieve. If not specified, retrieve all of the
     *                     preferences listed in the $prefs hash.
     *
     * @return int         PREFS_OK on success, PREFS_ERROR_* on failure.
     */
    function retrieve($prefs = array())
    {
        /*
         * If a list of preferences to retrieve hasn't been provided in
         * $prefs, assume all preferences are desired.
         */
        if (count($prefs) == 0) {
            $prefs = $this->listAll();
        }
        if (!is_array($prefs) || (count($prefs) == 0)) return PREFS_ERROR;

        /* Attempt to pull the values from the session cache first. */
        if ($this->cacheLookup($prefs)) {
            return PREFS_OK;
        }

        /* If we're not already connected, invoke the connect() method. */
        if (!$this->connected) {
            if ($this->connect() != PREFS_OK) return PREFS_ERROR_CONNECT;
        }

        /* Build the SQL query. */
        $query = 'select scope, pref_name, pref_value from ';
        $query .= $this->params['table'] . ' '; 
        $query .= "where uid = '" . $this->db->quoteString($this->user);
        $query .= "' and (scope = '" . $this->db->quoteString($this->scope);
        $query .= "' or scope = 'horde') order by scope";

        /* Execute the query. */
        $result = $this->db->query($query);

        if (isset($result) && !DB::isError($result)) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (DB::isError($row)) return PREFS_ERROR_EMPTY;

            /* Set the requested values in the $this->prefs hash based
             * on the contents of the SQL result.
             *
             * Note that Prefs::setValue() can't be used here because
             * of the check for the "changeable" bit.  We want to
             * override that check when populating the $this->prefs
             * hash from the SQL server. */
            while ($row && !DB::isError($row)) {
                $name = trim($row['pref_name']);
                if (in_array($name, $prefs)) {
                    $this->prefs[$name]['val'] = $row['pref_value'];
                    $this->setDirty($name, false);
                }
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }
            $result->free();

        } else {
            return PREFS_ERROR;
        }

        /* Update the session cache. */
        $this->cacheUpdate();
        
        return PREFS_OK;
    }

    /**
     * Stores preferences to SQL server.
     *
     * @param array $prefs (optional) An array listing the preferences to be
     *                     stored.  If not specified, store all of the
     *                     preferences listed in the $prefs hash.
     *
     * @return boolean     PREFS_OK on success, PREFS_ERROR_* on failure.
     */
    function store($prefs = array())
    {
        /*
         * If a list of preferences to store hasn't been provided in
         * $prefs, assume all preferences are desired.
         */
        if (count($prefs) == 0) {
            $prefs = $this->listAll();
        }
        if (!is_array($prefs)) return PREFS_ERROR;

        /* Check for any "dirty" preferences. */
        $dirty_prefs = array();
        foreach ($prefs as $pref) {
            if ($this->isDirty($pref)) {
                $dirty_prefs[] = $pref;
            }
        }

        /*
         * If no "dirty" preferences were found, there's no need to update
         * the SQL server.  Exit successfully.
         */
        if (count($dirty_prefs) == 0) {
            return PREFS_OK;
        }

        /* If we're not already connected, invoke the connect() method. */
        if (!$this->connected) {
            if ($this->connect() != PREFS_OK) return PREFS_ERROR_CONNECT;
        }

        /*
         * Loop through the "dirty" preferences.  If our attempt to insert
         * a new row fails, try to update an existing one.
         */
        foreach ($dirty_prefs as $name) {
            $scope = $this->getScope($name);

            /* Attempt an insert. */
            $query  = 'insert into ' . $this->params['table'] . ' ';
            $query .= '(uid, scope, pref_name, pref_value) values';
            $query .= "('" . $this->db->quoteString($this->user) . "', '";
            $query .= $this->db->quoteString($scope) . "', '" . $this->db->quoteString($name) . "', '";
            $query .= $this->db->quoteString($this->getValue($name)) . "')";
            $result = $this->db->query($query);
            
            /* If the insert failed, attempt an update. */
            if ($result !== DB_OK) {
                $query = 'update ' . $this->params['table'] . ' ';
                $query .= "set pref_value = '" . $this->db->quoteString($this->getValue($name)) . "'";
                $query .= " where uid = '" . $this->db->quoteString($this->user) . "'";
                $query .= " and pref_name = '" . $this->db->quoteString($name) . "'";
                $query .= " and scope = '" . $this->db->quoteString($scope) . "'";
                $result = $this->db->query($query);
                
                /* Return an error if the update fails, too. */
                if ($result !== DB_OK) {
                    return PREFS_ERROR;
                }
            }

            /* Mark this preference as "clean" now. */
            $this->setDirty($name, false);
        }

        /* Update the session cache. */
        $this->cacheUpdate();

        return PREFS_OK;
    }


    /**
     * Perform cleanup operations.
     *
     * @param bool  $all    (optional) Cleanup all Horde preferences.
     */
    function cleanup($all = false)
    {
        /* Close the database connection. */
        if (isset($this->db) && is_object($this->db)) {
            $this->db->disconnect();
        }

        parent::cleanup($all);
    }
}
?>
