<?php
// $Horde: horde/lib/Registry.php,v 1.37 2001/06/06 17:20:13 chuck Exp $

require_once 'PEAR.php';

/**
 * The Registry:: class provides a set of methods for communication
 * between Horde applications and keeping track of application
 * configuration information.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.37 $
 * @since   Horde 1.3
 * @package horde
 */
class Registry {
    
    /**
     * Hash representing the actual registry structure.
     * @var array $registry
     */
    var $registry = array();

    /**
     * Hash storing all of the known services and callbacks.
     * @var array $services
     */
    var $services = array();

    /**
     * Hash storing information on each registry-aware application.
     * @var array $applications
     */
    var $applications = array();

    /**
     * Stack of in-use applications.
     * @var array $appStack
     */
    var $appStack = array();

    /**
     * Stack of saved $prefs objects
     * @var array $prefStack
     */
    var $prefStack = array();

    /**
     * List of implemented services.
     * @var array $used
     */
    var $used;
    
    /**
     * Returns a reference to the global Registry object, only
     * creating it if it doesn't already exist.
     *
     * This method must be invoked as: $registry = &Registry::singleton()
     *
     * @return object The Horde Registry instance.
     */
    function &singleton()
    {
        static $registry;

        if (!isset($registry)) {
            $registry = new Registry();
        }

        return $registry;
    }

    /**
     * Create a new registry instance.
     */
    function Registry()
    {
        // read in and evaluate the registry file
        include_once dirname(__FILE__) . '/../config/registry.php';
    }

    /**
     * Determine if guests should be allowed access to the specified
     * application. If no application is specified, check the current
     * application.
     *
     * @param string $app (optional) The application to check
     *
     * @return boolean Should guests be allowed?
     */
    function allowGuests($app = null)
    {
        if (!isset($app)) $app = $this->getApp();
        
        return (!empty($this->applications[$app]['allow_guests']));
    }

    /**
     * Determine if a method has been registered with the registry.
     * 
     * @param string $method The full name of the method to check for.
     * 
     * @return boolean Whether or not the method is registered.
     */
    function hasMethod($method)
    {
        list($type, $subtype) = explode('/', $method);
        return !empty($this->registry[$type][$subtype]);
    }

    /**
     * Returns the handler for the given method.
     * 
     * @param string $method The full name of the method to check for.
     * 
     * @return string The handler for the method.
     */
    function getMethod($method)
    {
        list($type, $subtype) = explode('/', $method);
        return !empty($this->registry[$type][$subtype]) ? $this->registry[$type][$subtype] : null;
    }

    /**
     * Return the hook corresponding to the default package that
     * provides the functionality requested by the $method
     * parameter. $method is a string consisting of
     * "packagetype/methodname".
     *
     * @param $method the method to call
     * @param $args (optional) arguments to the method
     */
    function call($method, $args = array())
    {
        if (!$this->hasMethod($method)) {
            // no callback defined
            return PEAR::raiseError('The method "' . $method . '" is not defined in the Horde registry,');
        }
        
        list($type, $subtype) = explode('/', $method);
        return $this->callByPackage($this->registry[$type][$subtype], $method, $args);
    }

    /**
     * Echo the output of call()
     *
     * @param $method the method to call
     * @param $args (optional) arguments to the method
     */
    function pcall($method, $args = array())
    {
        echo $this->call($method, $args);
    }

    /**
     * Output the hook corresponding to the specific package named.
     *
     * @param $package the desired package.
     * @param $method the method to call.
     * @param $args (optional) arguments to the method.
     */
    function callByPackage($package, $method, $args = array())
    {
        list($type, $subtype) = explode('/', $method);
        if (empty($this->services[$package][$type][$subtype]['file']) ||
            empty($this->services[$package][$type][$subtype]['function'])) {
            // Either the file or the function definition is missing
            return PEAR::raiseError('The method "' . $method . '" is not defined in the Horde registry,');
        }

        $file = $this->services[$package][$type][$subtype]['file'];
        $function = $this->services[$package][$type][$subtype]['function'];

        // Make sure we've got a full path to the file.
        if (isset($this->applications[$package]['fileroot'])) {
            $file = str_replace('%application%', $this->applications[$package]['fileroot'], $file);
        }

        if (!@is_readable($file)) {
            return PEAR::raiseError('The file defining the ' . $method . ' call (' . $file . ') is not readable.');
        }

        include_once $file;
        if (!function_exists($function)) {
            return PEAR::raiseError('The function implementing ' . $method . ' (' . $function . ') is not defined in ' . $file . '.');
        }

        $this->pushApp($package);
        $res = call_user_func_array($function, $args); 
        $this->popApp();

        return $res;
    }

    /**
     * Echo the output of callByPackage()
     *
     * @param $package the desired package
     * @param $method the method to call
     * @param $args (optional) arguments to the method
     */
    function pcallByPackage($package, $method, $args = array())
    {
        echo $this->callByPackage($package, $method, $args);
    }

    /**
     * Return the hook corresponding to the default package that
     * provides the functionality requested by the $method
     * parameter. $method is a string consisting of
     * "packagetype/methodname".
     *
     * @param $method the method to link to.
     * @param $args (optional) arguments to the method
     * @param $extra (optional) extra, non-standard arguments to the method
     */
    function link($method, $args = array(), $extra = array())
    {
        if (!$this->hasMethod($method)) {
            // no link defined
            return null;
        }
        
        list($type, $subtype) = explode('/', $method);
        return $this->linkByPackage($this->registry[$type][$subtype], $method, $args, $extra);
    }

    /**
     * Echo the output of link()
     *
     * @param $method the method to link to.
     * @param $args (optional) arguments to the method
     * @param $extra (optional) extra, non-standard arguments to the method
     */
    function plink($method, $args = array(), $extra = array())
    {
        echo $this->link($method, $args, $extra);
    }

    /**
     * Output the hook corresponding to the specific package named.
     *
     * @param $package the desired package.
     * @param $method the method to link to..
     * @param $args (optional) arguments to the method.
     * @param $extra (optional) extra, non-standard arguments to the method.
     */
    function linkByPackage($package, $method, $args = array(), $extra = array())
    {
        list($type, $subtype) = explode('/', $method);
        if (empty($this->services[$package][$type][$subtype]['link'])) {
            // no link defined
            return null;
        }

        // mark this service as used so that we can output support
        // functions at the end of the page
        $this->used[$method] = true;

        $link = $this->services[$package][$type][$subtype]['link'];

        // fill in html-encoded arguments
        foreach ($args as $key => $val) {
            $link = str_replace('%' . $key . '%', htmlentities($val), $link);
        }
        if (isset($this->applications[$package]['webroot'])) {
            $link = str_replace('%application%', $this->applications[$package]['webroot'], $link);
        }

        // repace urlencoded arguments that haven't been specified with an
        // empty string (this is where the default would be substituted in
        // a stricter registry).
        $link = preg_replace("|%.+%|U", '', $link);

        // fill in urlencoded arguments
        foreach ($args as $key => $val) {
            $link = str_replace('|' . $key . '|', urlencode($val), $link);
        }

        // append any extra, non-standard arguments
        $extra_args = '';
        foreach ($extra as $key => $val) {
            $key = urlencode($key);
            $val = urlencode($val);
            $extra_args .= "&$key=$val";
        }
        $link = str_replace('|extra|', $extra_args, $link);

        // repace html-encoded arguments that haven't been specified with
        // an empty string (this is where the default would be substituted
        // in a stricter registry).
        $link = preg_replace('|\|.+\||U', '', $link);

        return htmlspecialchars($link);
    }

    /**
     * Echo the output of linkByPackage()
     *
     * @param $package the desired package
     * @param $method the method to link to.
     * @param $args (optional) arguments to the method
     */
    function plinkByPackage($package, $method, $args = array())
    {
        echo $this->linkByPackage($package, $method, $args);
    }

    /**
     * Return the code corresponding to the default package necessary
     * to directly invoke the method. If you want to print results,
     * this will print the results, not a link to get the results. If
     * you want to invoke a javascript function, this will invoke it, not
     * print a link that when clicked will invoke it.
     *
     * @param $method the method to invoke.
     * @param $args (optional) arguments to the method.
     * @param $extra (optional) extra, non-standard arguments to the method.
     */
    function invoke($method, $args = array(), $extra = array())
    {
        list($type, $subtype) = explode('/', $method);
        return $this->invokeByPackage($this->registry[$type][$subtype], $method, $args, $extra);
    }

    /**
     * Echo the output of invoke()
     *
     * @param $method the method to invoke
     * @param $args (optional) arguments to the method
     * @param $extra (optional) extra, non-standard arguments to the method
     */
    function pinvoke($method, $args = array(), $extra = array())
    {
        echo $this->invoke($method, $args, $extra);
    }

    /**
     * Return the code corresponding to the default package necessary
     * to directly invoke the method. If you want to print results,
     * this will print the results, not a link to get the results. If
     * you want to invoke a javascript function, this will invoke it, not
     * print a link that when clicked will invoke it.
     *
     * @param $package the desired package.
     * @param $method the method to invoke.
     * @param $args (optional) arguments to the method.
     * @param $extra (optional) extra, non-standard arguments to the method.
     */
    function invokeByPackage($package, $method, $args = array(), $extra = array())
    {
        // mark this service as used so that we can output support
        // functions at the end of the page
        $this->used[$method] = true;
        list($type, $subtype) = explode('/', $method);

        if (empty($this->services[$package][$type][$subtype]['invocation'])) {
            // no invocation defined
            return null;
        }

        $invocation = $this->services[$package][$type][$subtype]['invocation'];

        // fill in urlencoded arguments
        foreach ($args as $key => $val) {
            $invocation = str_replace('%' . $key . '%', htmlentities($val), $invocation);
        }
        $invocation = str_replace('%application%', $this->applications[$package]['webroot'], $invocation);

        // replace urlencoded arguments that haven't been specified
        // with an empty string (this is where the default would be
        // substituted in a stricter registry).
        $invocation = preg_replace("|%.+%|U", '', $invocation);

        // fill in html-encoded arguments
        foreach ($args as $key => $val) {
            $invocation = str_replace('|' . $key . '|', urlencode($val), $invocation);
        }

        // append any extra, non-standard arguments
        $extra_args = '';
        foreach ($extra as $key => $val) {
            $key = urlencode($key);
            $val = urlencode($val);
            $extra_args .= "&$key=$val";
        }
        $invocation = str_replace('|extra|', $extra_args, $invocation);

        // repace html-encoded arguments that haven't been specified with
        // an empty string (this is where the default would be substituted
        // in a stricter registry).
        $invocation = preg_replace('|\|.+\||U', '', $invocation);
        
        return $invocation;
    }

    /**
     * Echo the output of invokeByPackage().
     *
     * @param $package the desired package.
     * @param $method the method to invoke.
     * @param $args (optional) arguments to the method.
     */
    function pinvokeByPackage($package, $method, $args = array())
    {
        echo $this->invokeByPackage($package, $method, $args);
    }

    /**
     * Go through the list of methods that have been used, and
     * evaluate/include any support files needed.
     */
    function shutdown()
    {
        if (!empty($this->used) && is_array($this->used)) {
            foreach ($this->used as $key => $val) {
                list($type, $subtype) = explode('/', $key);
                $application = $this->registry[$type][$subtype];
                if (!empty($this->services[$application][$type][$subtype]['includeFile'])) {
                    include_once $this->applicationFilePath($this->services[$application][$type][$subtype]['includeFile'], $application);
                }
            }
        }
    }

    /**
     * Replace any %application% strings with the filesystem path to
     * the application.
     */
    function applicationFilePath($path, $app = null)
    {
        if (!isset($app)) $app = $this->getApp();
        
        return str_replace('%application%', $this->applications[$app]['fileroot'], $path);
    }

    /**
     * Replace any %application% strings with the web path to the
     * application.
     */
    function applicationWebPath($path, $app = null)
    {
        if (!isset($app)) $app = $this->getApp();
        
        return str_replace('%application%', $this->applications[$app]['webroot'], $path);
    }

    /**
     * Set the current application, adding it to the top of the Horde
     * application stack.
     *
     * pushApp() also reads the application's configuration file and
     * sets up its global $conf hash.
     *
     * @param string $app The name of the application to push.
     */
    function pushApp($app)
    {
        if ($app != $this->getApp()) {
            if (isset($GLOBALS['prefs']) && is_object($GLOBALS['prefs'])) {
                array_push($this->prefStack, $GLOBALS['prefs']);
            }
            array_push($this->appStack, $app);
            
            /* Import this application's configuration values and set
               the gettext domain. */
            bindtextdomain($app, $this->applications[$app]['fileroot'] . '/locale');
            textdomain($app);
            $this->importConfig($app);
        }
    }

    /**
     * Remove the current app from the application stack, setting the
     * current app to whichever app was current before this one took
     * over.
     *
     * @return string The name of the application that was popped.
     */
    function popApp()
    {
        /* Pop the current application off of the stack. */
        $old_app = array_pop($this->appStack);

        /* Import the new active application's configuration values
           and set the gettext domain. */
        if ($this->getApp()) {
            $GLOBALS['prefs'] = array_pop($this->prefStack);
            bindtextdomain($this->getApp(), $this->applications[$this->getApp()]['fileroot'] . '/locale');
            textdomain($this->getApp());
            $this->importConfig($this->getApp());
        }

        return $old_app;
    }

    /**
     * Return the current application - the app at the top of the
     * application stack.
     *
     * @return string The current application.
     */
    function getApp()
    {
        if (count($this->appStack) > 0) {
            return $this->appStack[count($this->appStack) - 1];
        } else {
            return null;
        }
    }

    /**
     * Reads the configuration values for the given application and
     * imports them into the global $conf variable.
     *
     * @param string $app       The name of the application.
     */
    function importConfig($app)
    {
        $conf = array();
        include dirname(__FILE__) . '/../config/horde.php';
        include $this->applications[$app]['fileroot'] . '/config/conf.php';
        $GLOBALS['conf'] = $conf;
    }

}
?>
