<?php
// $Horde: horde/lib/Token.php,v 1.1 2001/05/17 02:58:12 chuck Exp $

/* Return codes */
define('CONN_OK',               0);    // Operation succeeded
define('CONN_ERROR',           -1);    // operation failed
define('CONN_ERROR_PARAMS',    -2);    // Bad or missing parameters ($params)
define('CONN_ERROR_CONNECT',   -3);    // Connection failure
define('CONN_ERROR_AUTH',      -4);    // Authentication failure
define('CONN_ERROR_EMPTY',     -5);    // Empty retrieval result

/**
 * The Token:: class provides a common abstracted interface into the
 * various token generation mediums. It also includes all of the
 * functions for retrieving, storing, and checking tokens.
 *
 * @author  Max Kalika <max@horde.org>
 * @version $Revision: 1.1 $
 * @since   Horde 1.3
 * @package horde.connection
 */
class Token {
    
    /**
     * Attempts to return a concrete Token instance based on $driver.
     * 
     * @param $driver   The type of concrete Token subclass to return.
     *                  This is based on the storage driver ($driver).  The
     *                  code is dynamically included.
     *
     * @param $params   (optional) A hash containing any additional
     *                  configuration or connection parameters a subclass
     *                  might need.
     *
     * @return          The newly created concrete Token instance, or
     *                  false an error.
     */
    function factory($driver, $params = array())
    {
        $driver = strtolower($driver);
        if (empty($driver) || (strcmp($driver, 'none') == 0)) {
            return new Token;
        }

        if (@include_once dirname(__FILE__) . '/Token/' . $driver . '.php') {
            $class = 'Token_' . $driver;
            return new $class($params);
        } else {
            return false;
        }
    }

    function hexRemoteAddr()
    {
        global $HTTP_SERVER_VARS;
        
        $addr = explode('.', $HTTP_SERVER_VARS['REMOTE_ADDR']);
        return sprintf("%02x%02x%02x%02x", $addr[0], $addr[1], $addr[2], $addr[3]);
    }
    
    /**
     * Generates a connection id and returns it.
     *
     * @return string   The generated id string.
     */
    function generateID()
    {
        return md5(time() . '][' . Token::hexRemoteAddr());
    }
    
    /**
     * Checks if the given token has been previously used.  First
     * purges all expired tokens. Then retreives current tokens for
     * the given ip address. If the specified token was not found,
     * adds it.
     *
     * @param $token      The value of the token to check.
     *
     * @return bool       True if the token has not been used,
     *                    false otherwise.
     */
    function verify($token)
    {
        $this->purge();
        if ($this->exists($token)) {
            return false;
        }
        else {
            $this->add($token);
            return true;
        }
    }

    /**
     * This is basically an abstract method that should be overridden by a
     * subclass implementation. It's here to retain code integrity in the
     * case that no subclass is loaded ($driver == 'none').
     */
    function exists()
    {
        return false;
    }

    /**
     * This is basically an abstract method that should be overridden by a
     * subclass implementation. It's here to retain code integrity in the
     * case that no subclass is loaded ($driver == 'none').
     */
    function add()
    {
        return true;
    }

    /**
     * This is basically an abstract method that should be overridden by a
     * subclass implementation. It's here to retain code integrity in the
     * case that no subclass is loaded ($driver == 'none').
     */
    function purge()
    {
        return true;
    }
}
?>
